<?php
/**
 * This is a trimmed front end just for real time playing. It will print
 * only the move description
 */
require("mcc_common.php");
require("mcc_scripts.php");

$current_player = mcc_check_login();

$username       = mcc_get_page_parameter("player");
$move_time      = mcc_get_page_parameter("mvtime", -1);
$gameid         = mcc_get_page_parameter("gameid");
$boardview_mode = mcc_get_page_parameter("boardview", BOARDVIEW_STANDARD);

$current_player = new Player($username);

try {
	$current_game = new Game($gameid);
} catch ( Exception $e ) {
	$current_game = NULL;
}

if ( !$current_game ) {
	echo 'Error not in a Game';
}

$player_w = $current_game->getWhitePlayer();
$player_b = $current_game->getBlackPlayer();


// If nobody connected, we want to see the whites down

if ( $current_player != $player_b ) {
	$player_opp = $player_b;
	$player_color = "w";
}
else {
	$player_opp = $player_w;
	$player_color = "b";
}

/* MCC Note: These are modified by handle_move.php functions... This behaviour should
 *           change ASAP...   */

$browsing_mode = 0;
$ac_move = "";
$w_figures = array();
$b_figures = array();
$attackers = array();
$board = array(
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
	"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  " );

$chessmove      = mcc_get_page_parameter("chessmove");
$chesscomment   = mcc_get_page_parameter("chesscomment");
$drawoffer      = mcc_get_page_parameter("drawoffer");
$cmd_move       = strlen($chessmove) > 0;
$cmd_draw       = mcc_get_page_parameter("accept_draw");
$cmd_delete     = mcc_get_page_parameter("delete_game");
$cmd_undo       = mcc_get_page_parameter("undo_move");
$cmd_archive    = mcc_get_page_parameter("move_to_archive");
$cmd_savenote   = mcc_get_page_parameter("refresh_and_save_note");
define("NULLNOTE", "=== No note ====");
$chessnote      = mcc_get_page_parameter("chessnote", NULLNOTE);

if ( $drawoffer ) {
	$drawoffer = 1;
}
else {
	$drawoffer = 0;
}

// Hack to avoid problems with game history and positional draw
if ( $cmd_move ) {
	$current_game->purgeLegacyVariablesCache();
}

$game = $current_game->initializeLegacyVariables();

/* ***** COMPUTE WHETHER UNDO OKAY ***** */
// The first time, we test if we can validate an undo submission

$lastmoveseconds = mcc_db_time() - $current_game->getLastMoveDate();

// no undo move on real time games
$may_undo = 0;

/* ***** CHECK A SUBMITTED MOVE OR UNDO ***** */

if ( $cmd_delete ) {
	if ( $current_player != $player_w && $current_player != $player_b ) {
		$move_result = "ERROR: You do not participate in this game!";
	} else if ( $current_game->getNextPlayer() == $current_player ) {
		$move_result = "ERROR: You cannot delete a game, if it is your turn!";
	} else if ( ! $current_game->isOpen() ) {
		$move_result = "ERROR: Only open games may be deleted!";
	} else if ( $current_game->isDeleted() ) {
		$move_result = "ERROR: Game already deleted !";
	} else if ( $lastmoveseconds < DROP_DELAY_MINS * 60 ) {
		$move_result = "ERROR: Your opponents move must be two weeks or older to delete the game!";
	} else {
		$move_result = "Game deleted.";
		$pl_w = $current_game->getWhitePlayer();
		$pl_b = $current_game->getBlackPlayer();

		$player_is_w = ( $pl_w == $current_player );
		$player_is_b = ( $pl_b == $current_player );

		if ( ($player_is_w && $current_game->getMoveCount() >= 2) ||
				 ($player_is_b && $current_game->getMoveCount() >= 1 ) ) {

			if ( $player_is_w ) {
				updateStats( $pl_w->getIdentifier(), $pl_b->getIdentifier(), 'w' );
			} else {
				updateStats( $pl_w->getIdentifier(), $pl_b->getIdentifier(), 'b' );
			}
		}

		$current_game->delete();
	}
}
else if ( $cmd_undo ) {
	// $move_result = handleUndo( $game );
	if ( $may_undo ) {
		$current_game->performUndo();
		$move_result = "Move " . trim($ac_move, "?") . " undone !";
		$game = $current_game->initializeLegacyVariables();
	}
	else {
		$move_result = "You cannot undo !";
	}
}
else if ( $cmd_draw ) {
	if ( $current_game->isDrawOffered() ) {
		$current_game->draw();
		$pl_w = $current_game->getWhitePlayer();
		$pl_b = $current_game->getBlackPlayer();
		updateStats($pl_w->getIdentifier(), $pl_b->getIdentifier(), '-');
	}
}
else {
	if ( $cmd_archive ) {
		/* only allow if user is active (which it may not be if
			 undo function was used) */

		LogDebug("move_to_archive is set...");

		if ( ! $current_game->isOpen() ) {
			LogDebug("moving to archive !");
			$current_game->setArchived(TRUE);
		}
	}

	if ( $cmd_move && !empty($chessmove) && $current_game->getNextPlayer() == $current_player ) {
		$move_result = handleMove($game, $board, $w_figures, $b_figures, $username,
				$ac_move, $res_games, $chessmove, $chesscomment, $current_game);
	}

	if ( $chessnote != NULLNOTE && $current_game->hasPlayer($current_player) ) {
		$note = new Note($current_player, $current_game);
		$note->setText($chessnote);
	}
}

if ( isset($move_result) ) {
    $current_game->updateFromLegacyVariables($game, $move_result, $chesscomment, $drawoffer, $move_time);
}

// on real time games can not undo
$may_undo = 0;

$move_id     = $current_game->getTurnCount();
$move_player = $current_game->getNextPlayer();
$move_color  = ($move_player == $player_w)?"White":"Black";
$current_player_color = ($current_player == $player_w)?"White":"Black";

$disp_move_id = $move_id;

if ( $move_player == $player_w ) {
	$disp_move_id++;
}

$name_w = $player_w->getIdentifier();
$name_b = $player_b->getIdentifier();

$is_playing = 0;

if ( $current_player == $move_player && $current_game->isOpen() ) {
	$is_playing = 1;
}

/* ***** MOVE DESCRIPTION ***** */

$last_move = $current_game->getLastMove();

// the very purpose of this script, process a move and print its
// long description
if ( $current_game->isOpen() && $last_move ) {
    echo $last_move->getLong();
}
