var socket = io.connect(config.nodejs_server_url),
    online_players;

/**
 * This notify nodejs server that this user has been connected
 * current player must be defined on earlier pages
 */
socket.emit('player connected', current_player);

/**
 * Setting the session on server
 */
socket.on('reconnect', function(){
    // send session data on reconnection
    socket.emit('player connected', current_player);
});


/**
 * This update list of online players, burden presence server implementation
 */
socket.on('update online players', function (data) {
    // if in page club.php
    if(typeof update_online_players == 'function') {
        update_online_players(data);
    }

    // for all pages
    online_players = [];
    for(var i = 0; i < data.length; i += 1) {
        online_players.push(data[i].pl_identifier);
    }
});

/**
 * This show invitation
 */
socket.on('go real time', function (data, callback) {
    var url = 'http://' + window.location.host +
            '/chessrt.php?gameid=' + data.game +
            '&invited=1&time=' + data.time,
        msg = 'User ' + data.inviter + ' wants to play a game in real time mode ';

    if(data.time === 0){
        msg += 'without time limit. Do you accept?';
    } else {
        msg += 'to ' + data.time + ' minutes . Do you accept?';
    }
    if(confirm(msg)) {
        callback('ok');
        window.location.href = url;
    } else {
        callback(false);
    }

});

function invite_opponent(data, callback){
    var time = data.time,
        url = 'http://' + window.location.host +
            '/chessrt.php' +
            window.location.search +
            '&invited=0&time=' + time;

    socket.emit('go real time', data,
        function(ack){
            console.log('Receive ack: ' + ack);
            if(ack == 'ok'){
                callback(true);
                window.location.href = url;
            } else {
                callback(false);
            }
        }
    );
}

function invite_buttons_callback(dialog, time) {
    timer = setTimeout(function(){
        $( "#progressbar" ).hide();
        $( dialog ).dialog( "close" );
        alert('Opponent not online or did not receive the invitation.');
    }, config.user_invitation_timeout);
    invite_opponent(
        // TODO this data is from global space, this should be changed
        {player: player_opp, inviter: current_player, time: time, game: gameid},
        function(accepted){
            clearTimeout(timer);
            $( dialog ).dialog( "close" );
            if(! accepted) {
                alert('Your opponent rejected the invitation.');
            }
    });
}

$(document).ready(function(){

    $( document ).tooltip({ track: true });
    $('#invite-dialog').hide();
    $( "#progressbar" ).progressbar({ value: false }).hide();

    $('#invite-button').click(function(){
        $('#invite-dialog').dialog({
            title: "Start a real time game",
            width: 500,
            heigh: 300,
            modal: true,
            resizable: false,
            buttons: {
                "Go Real Time !!": function(){
                    var time = $( "#select-time" ).slider( "value" );
                    $( "#progressbar" ).show();
                    invite_buttons_callback(this, time);
                 },
                "No Clock": function(){
                    $( "#progressbar" ).show();
                    invite_buttons_callback(this, 0);
                },
                'Stay quiet': function() {
                    $( this ).dialog( "close" );
                }
            },
            close: function(){
                $( "#progressbar" ).hide();
            }
        });
    });

    // create the slider
    $( "#select-time" ).slider({
        min: 2,
        max: 60,
        value: 5,
        range: "min",
        slide: function( event, ui ) {
            $( "#amount" ).val( ui.value + ' min' );
        }
    });
    // set up initial value for slider
    $( "#amount" ).val( $( "#select-time" ).slider( "value" ) + ' min' );

});
