<?php

class Move {
	public $_db_fields;

	function Move ( $p_fields ) {
		$this->_db_fields = $p_fields;
	}

	function getId() {
		return $this->_db_fields['mv_id'];
	}

	function getShort() {
		return $this->_db_fields['mv_short'];
	}

	function getLong() {
		return $this->_db_fields['mv_long'];
	}

	function getChatter() {
		return $this->_db_fields['mv_chat'];
	}

	function isAnalysisAvailable() {
		return $this->_db_fields['mv_score'] != NULL;
	}

	function getTeacherMove() {
		$pv = $this->_db_fields['mv_teachermove'];
		$offs_space = strpos($pv, ' ');

		if ( $offs_space == FALSE ) {
			$move = $pv;
		}
		else {
			$move = substr($pv, 0, $offs_space);
		}

		return $move;
	}

	function getTeacherPrincipalVariation() {
		return $this->_db_fields['mv_teachermove'];
	}

	function getScore() {
		return $this->_db_fields['mv_score'];
	}

	function getTeacherRate() {
		return $this->_db_fields['mv_teacherrate'];
	}

	function getDate() {
		return $this->_db_fields['mv_stamp'];
	}

	function getSecondsSinceMove() {
		$date = $this->getDate();
		$result = mcc_db_time() - $date;

		LogDebug("getSecondsSinceMove = $result");
		return $result;
	}

    function getTimer() {
        return $this->_db_fields['mv_timer'];
    }
};

class Game {
	public $game_id;
	public $game_board_history;
	public $_att_cache;
	public $_moves_cache;

	public $STATUS_OPEN   = 'open';

	public $WINNER_WHITE  = 'whitewon';
	public $WINNER_BLACK  = 'blackwon';
	public $WINNER_DRAW   = 'draw';

	function Game ( $p_gameid, $p_white = NULL, $p_black = NULL ) {
		global $mysql_table_game;

		$this->game_board_history = array();
		$this->_att_cache         = NULL;
		$this->game_id            = $p_gameid;
		$this->deleted            = FALSE;
		$this->_moves_cache       = NULL;

		if ( $p_white && $p_black && $p_white != $p_black ) {

			// We create a new game...

			$white_id = $p_white->getIdentifier();
			$black_id = $p_black->getIdentifier();

			$result = sql_exec("insert into $mysql_table_game set "
				. " gm_player_white='$white_id',"
				. " gm_player_black='$black_id',"
				. " gm_status='{$this->STATUS_OPEN}',"
				. " gm_date_start=now()");

			$this->game_id = $result[0]->lastInsertId();

			if ( $result[1]->rowCount() != 1 ) {
				throw new Exception("Game database insertion failed");
			}
		}
		else {
			// We check if p_gameid is a valid id...

			$q = sql_query("select gm_id from $mysql_table_game"
					. " where gm_id='$p_gameid'");

			if ( empty($q) ) {
				throw new Exception("Could not fetch game from database");
			}
		}
	}

	function getId() {
		return $this->game_id;
	}

	function getWhitePlayer() {
		return new Player($this->getGameAttribute('gm_player_white'));
	}

	function getBlackPlayer() {
		return new Player($this->getGameAttribute('gm_player_black'));
	}

	function getOpponent ( $p_player ) {
		if ( $this->getGameAttribute('gm_player_white') == $p_player->getIdentifier() ) {
			$result = new Player($this->getGameAttribute('gm_player_black'));
		}
		else {
			$result = new Player($this->getGameAttribute('gm_player_white'));
		}

		return $result;
	}

	function hasPlayer ( $p_player ) {
		$id = $p_player->getIdentifier();
		return $this->getGameAttribute('gm_player_white') == $id
		    || $this->getGameAttribute('gm_player_black') == $id;
	}

	function getStartDate() {
		return $this->getGameAttribute('UNIX_TIMESTAMP(gm_date_start)');
	}

	function isDrawOffered() {
		$result = 0;

		$this->loadMovesInCache();

		$nbmoves = count($this->_moves_cache);

		if ( $nbmoves > 0 ) {
			$last   = $this->_moves_cache[count($this->_moves_cache) - 1];
			$result = $last['mv_drawoffer'];
		}

		LogDebug("isDrawOffered = $result");

		return $result;
	}

	function getLastNotificationDate() {
		return $this->getGameAttribute('UNIX_TIMESTAMP(gm_date_notification)');
	}

	function setLastNotificationDate ( $p_date = 'now()' ) {
		return $this->setGameAttribute('gm_date_notification', $p_date);
	}

	// This is a PRIVATE method !
	function dropMovesCache() {
		$this->_moves_cache = NULL;
	}

	// This is a PRIVATE method !
	function loadMovesInCache() {
		global $mysql_table_move;

		if ( ! $this->_moves_cache ) {
			$gid = $this->game_id;

			LogDebug("loadMovesInCache: Loading moves for game $gid");

			$this->_moves_cache = array();

			$q = sql_query("select UNIX_TIMESTAMP(mv_date) as mv_stamp,"
					. " mv_short, mv_long, mv_chat, mv_drawoffer,"
					. " mv_teachermove, mv_teacherrate, mv_score, mv_timer"
					. " from $mysql_table_move"
					. " where mv_game='$gid' order by mv_id");

			foreach($q as $row) {
				LogDebug("  - Move loaded: " . implode(", ", $row));
				$this->_moves_cache[] = $row;
			}
		}
	}

	function isOpen() {
		return (   $this->getGameAttribute('gm_status') == $this->STATUS_OPEN
			&& $this->getGameAttribute('gm_is_deleted') == 0 );
	}

	function setArchived ( $p_value ) {
		return $this->setGameAttribute('gm_is_archived', ($p_value)?1:0);
	}

	function isArchived() {
		$result = $this->getGameAttribute('gm_is_archived');
		LogDebug("isArchived() = $result");
		return $result == 1;
	}

	function getWinner() {
		return $this->getGameAttribute('gm_status');
	}

	function setWinner ( $p_winner ) {
		return $this->setGameAttribute('gm_status', $p_winner);
	}

	function getNextPlayer() {
		$result = NULL;

		$count = $this->getMoveCount();

		if ( fmod($count, 2) == 0 ) {
			$result = $this->getWhitePlayer();
		}
		else {
			$result = $this->getBlackPlayer();
		}

		return $result;
	}

	function getMoveCount() {
		$this->loadMovesInCache();
		$result = count($this->_moves_cache);

		LogDebug("getMoveCount() = $result");

		return $result;
	}

	function getTurnCount() {
		$result = floor(($this->getMoveCount() + 1) / 2);
		return $result;
	}

	function isRealtime() {
		$this->loadMovesInCache();
		if ( count($this->_moves_cache) < 2 ) {
			$basetime = $this->getStartDate();
		}
		else {
			$last2row = $this->_moves_cache[count($this->_moves_cache) - 2];
			$last2move = new Move($last2row);
			$basetime = $last2move->getDate();
		}

		$now = mcc_db_time();

		return $now - $basetime < 900;
	}

	function getMoves () {
		$this->loadMovesInCache();
		$result = array();

		foreach ( $this->_moves_cache as $movedata ) {
			$result[] = new Move($movedata);
		}

		return $result;
	}

	function getLastMoveDate() {
		$result = FALSE;
		$lastmove = $this->getLastMove();

		if ( $lastmove ) {
			$result = $lastmove->getDate();
		}
		else {
			$result = $this->getStartDate();
		}

		return $result;
	}

	function getLastMove() {
		$this->loadMovesInCache();

		$nbmoves = count($this->_moves_cache);
		$result  = NULL;

		if ( $nbmoves > 0 ) {
			$last   = $this->_moves_cache[count($this->_moves_cache) - 1];
			$result = new Move($last);
		}

		return $result;
	}

    function getBeforeLastMove() {
        $this->loadMovesInCache();

        $nbmoves = count($this->_moves_cache);
        $result  = NULL;

        if ( $nbmoves > 1 ) {
            $beforelast   = $this->_moves_cache[count($this->_moves_cache) - 2];
            $result = new Move($beforelast);
        }

        return $result;
    }

	function getAsPgn() {
		global $mcc_server_name;
		global $_SERVER;

		$dt = date( "Y.m.d", $this->getStartDate());
		$move_count = $this->getTurnCount();
		$white = $this->getWhitePlayer();
		$black = $this->getBlackPlayer();

		$winner = $this->getWinner();

		switch ( $winner ) {
			case $this->WINNER_WHITE: $pgnresult = "1-0";     break;
			case $this->WINNER_BLACK: $pgnresult = "0-1";     break;
			case $this->WINNER_DRAW:  $pgnresult = "1/2-1/2"; break;
			default:		  $pgnresult = "*";       break;
		}

		$result  = "";
		$result .= "[Event \"$mcc_server_name game\"]\n";
		$result .= "[Site \"". $_SERVER['SERVER_NAME'] ."\"]\n";
		$result .= "[Date \"$dt\"]\n";
		$result .= "[Round \"$move_count\"]\n";
		$result .= "[White \"". $white->getIdentifier()."\"]\n";
		$result .= "[Black \"". $black->getIdentifier()."\"]\n";
		$result .= "[Result \"$pgnresult\"]\n";

		$moves = $this->getMoves();

		$idx = 0;

		foreach ( $moves as $m ) {
			if ( ($idx % 10) == 0 ) {
				$result .= "\n";
			}

			if ( ($idx % 2) == 0 ) {
				$result .= ($idx/2 + 1) . ". ";
			}

			$result .= $m->getShort() . " ";
			$idx++;
		}

		$result .= "\n";

		return $result;
	}

	function addMove ( $p_short, $p_long, $p_chat, $p_drawoffer, $p_time ) {
		global $mysql_table_move;
		$gid = $this->game_id;

		if ( strlen($p_chat) > MAX_TEXT_LENGTH ) {
			$p_chat = substr($p_chat, 0, MAX_TEXT_LENGTH);
		}

		$p_chat = stripslashes($p_chat);
		//$p_chat = mysql_escape_string($p_chat);

		$result = NULL;
	    if ( $p_chat ) {
			$result = sql_exec("insert into $mysql_table_move"
					. " set mv_game=?, mv_short=?,"
					. " mv_long=?,"
					. " mv_drawoffer=?,"
					. " mv_date=now(), mv_chat=?, mv_timer=?",
                    array($gid, $p_short, $p_long, $p_drawoffer, $p_chat, $p_time));
		}
		else {
			$result = sql_exec("insert into $mysql_table_move"
					. " set mv_game=?, mv_short=?,"
					. " mv_long=?,"
					. " mv_drawoffer=?,"
					. " mv_date=now(), mv_timer=?",
                    array($gid, $p_short, $p_long, $p_drawoffer, $p_time));
		}

		$this->dropMovesCache();
		$this->purgeLegacyVariablesCache();

		return $result[1]->rowCount() == 1;
	}

	function performUndo() {

		// Warning: Potential deadlock -> This should be atomic...
		//	  We could use delete from ... order by ... limit 1, but
		//	  this would break MySQL 3.X compatibility.

		global $mysql_table_move;

		$gid    = $this->game_id;
		$result = FALSE;

		$q = sql_query("select mv_id from $mysql_table_move"
				. " where mv_game='$gid' order by mv_id desc limit 1");

		if ( ! empty($q) ) {
			$row = array_shift($q);
			$mid = $row['mv_id'];
			sql_exec("delete from $mysql_table_move where mv_id=?", array($mid));
			$result = TRUE;
		}

		$this->dropMovesCache();
		$this->purgeLegacyVariablesCache();

		return $result;
	}

	function draw() {
		$this->addMove('draw', 'Draw accepted', NULL, 0, -1);
		$this->purgeLegacyVariablesCache();
		return $this->setWinner($this->WINNER_DRAW);
	}

	function resignation ( $p_player ) {
		if ( $p_player == $this->getWhitePlayer() ) {
			return $this->setWinner($this->WINNER_BLACK);
		}

		if ( $p_player == $this->getBlackPlayer() ) {
			return $this->setWinner($this->WINNER_WHITE);
		}
		$this->purgeLegacyVariablesCache();
	}

	function delete() {
		$this->purgeLegacyVariablesCache();
		return $this->setGameAttribute('gm_is_deleted', 1);
	}

	function isDeleted() {
		return $this->getGameAttribute('gm_is_deleted') == 1;
	}

	function getCache() {
		global $mysql_table_game_cache;
		$result = NULL;

		$res = sql_query("select gc_cache from $mysql_table_game_cache"
			. " where gc_game=?", array($this->game_id));

		if ( $res && ($row = array_shift($res)) ) {
			$result = $row['gc_cache'];
		}

		return $result;
	}

	function setCache ( $p_data ) {
		global $mysql_table_game_cache;
		sql_query("replace into $mysql_table_game_cache"
			. " set gc_game=?,"
			. " gc_cache=?", array($this->game_id, $p_data));
	}

	//
	// ---  These functions are a cache to the legacy method...
	//

	function initializeLegacyVariablesFromCache() {
		global $board, $ac_move, $w_figures, $b_figures;
		$cache = $this->getCache();
		$game  = NULL;

		if ( $cache != "" ) {
			$gamedata  = unserialize($cache);
			$game      = $gamedata[0];
			$board     = $gamedata[1];
			$ac_move   = $gamedata[2];
			$w_figures = $gamedata[3];
			$b_figures = $gamedata[4];
			$this->game_board_history = $gamedata[5];
		}

		LogDebug("initializeLegacyVariablesFromCache()");
		LogDebug("Cache: $cache");

		return $game;
	}

	function storeLegacyVariablesInCache ( $p_game ) {
		global $board, $ac_move, $res_games, $w_figures, $b_figures;
		$gamedata = array($p_game, $board, $ac_move,
					$w_figures, $b_figures,
					$this->game_board_history);
		$this->setCache(serialize($gamedata));
	}

	// This must be called every time we alter the moves list
	function purgeLegacyVariablesCache() {
		$this->setCache('');
	}

	function initializeLegacyVariables() {
		global $board, $ac_move, $res_games, $w_figures, $b_figures;

		$game = $this->initializeLegacyVariablesFromCache();

		if ( $game ) {
			return $game;
		}

		// Note that username is not modified... We don't want to
		// make any confusion between our local var and the true
		// global one.

		LogDebug("======== initializeLegacyVariables =============");

		$player_w = $this->getWhitePlayer();
		$player_b = $this->getBlackPlayer();

		$id_w = $player_w->getIdentifier();
		$id_b = $player_b->getIdentifier();

		$game   = array();
		$game[] = "DUMMY DATES";
		$game[] = "$id_w $id_b 0 w ? 1 1 1 1 x x x x\n";
		$game[] = "Ra1 Nb1 Bc1 Qd1 Ke1 Bf1 Ng1 Rh1 Pa2 Pb2 Pc2 Pd2 Pe2 Pf2 Pg2 Ph2\n";
		$game[] = "Ra8 Nb8 Bc8 Qd8 Ke8 Bf8 Ng8 Rh8 Pa7 Pb7 Pc7 Pd7 Pe7 Pf7 Pg7 Ph7\n";
		$game[] = "no chatter";

		$board = array(
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
			"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  " );

		fillChessBoard($board, $w_figures, $b_figures, $game[2], $game[3]);

		$this->game_board_history   = array();
		$this->game_board_history[] = $board;

		$mymoves = $this->getMoves();

		$username = $id_w;
		$color    = "w";

		foreach ( $mymoves as $m ) {
			//$err = completeMove($color, $m);
			//LogDebug("$m ==> $ac_move (err is $err)");
			$ac_move = $m->getShort();

			if ( substr($ac_move, -1) == "+" ) {
				$ac_move = substr($ac_move, 0, -1);
			}

			if ( $ac_move == "stalemate" || $ac_move == "mate"
				|| $ac_move == "resigned" || $ac_move == "positional" ) {
				continue;
			}

			$move_res = handleMove( $game, $board, $w_figures, $b_figures,
						$username, $ac_move, $res_games, $ac_move,
						"", $this, FALSE );

			LogDebug("Game status AFTER handleMove()");
			foreach ( $game as $gameline ) {
				LogDebug("game: " . trim($gameline));
			}

			LogDebug("board = " . implode(", ", $board));
			LogDebug("w_figures = " . implode(", ", $w_figures));
			LogDebug("b_figures = " . implode(", ", $b_figures));
			LogDebug("MOVE: $ac_move - $move_res");

			if ( $username == $id_w ) {
				$username = $id_b;
				$color    = "b";
			}
			else {
				$username = $id_w;
				$color    = "w";
			}

			LogDebug("====================================================");
		}

		$this->storeLegacyVariablesInCache($game);

		return $game;
	}

	function updateFromLegacyVariables ( $p_game, $p_move_result, $p_chat, $p_drawoffer, $move_time = -1 ) {
		LogDebug("======== updateFromLegacyVariables =============");
		foreach ( $p_game as $gameline ) {
			LogDebug("game: " . trim($gameline));
		}

		$headline = explode(' ', $p_game[1]);

		$moves = $this->getMoves();
		$moveidx = 0;

		foreach ( array_slice($p_game, 4, $headline[2]) as $gameline ) {
			$moveitems = explode(" ", trim($gameline));

			foreach ( array_slice($moveitems, 1) as $move ) {
				if ( $moveidx >= count($moves) ) {
					if ( $move == 'draw' ) {
						// do nothing
					}
					else if ( $move == 'stalemate' ) {
						$this->setWinner($this->WINNER_DRAW);
					}
					else if ( $move == 'mate' || $move == 'resigned' ) {
						if ( $moveidx % 2 == 1 ) {
							$this->setWinner($this->WINNER_WHITE);
						}
						else {
							$this->setWinner($this->WINNER_BLACK);
						}

						if ( $move == 'resigned' ) {
							$this->addMove($move, $p_move_result, $p_chat, $p_drawoffer, $move_time);
						}
					}
					else {
						$this->addMove($move, $p_move_result, $p_chat, $p_drawoffer, $move_time);
					}

					$p_chat = NULL;	// If we have 2 entries, chat only once
				}

				$moveidx++;
			}
		}

		$this->storeLegacyVariablesInCache($p_game);
	}

	// --- Internal API

	function getGameAttribute ( $p_attname ) {
		global $mysql_table_game;
		$result = NULL;

		if ( $this->_att_cache == NULL ) {
			LogDebug("getGameAttribute($p_attname)  // Cache reload");

			$id = $this->game_id;

			$q = sql_query("select *,UNIX_TIMESTAMP(gm_date_start),"
						. "UNIX_TIMESTAMP(gm_date_notification)"
					. " from $mysql_table_game"
					. " where gm_id=?", array($id));

			if ( ! empty($q) ) {
				$this->_att_cache = array_shift($q);
			}
		}

		$result = $this->_att_cache[$p_attname];

		LogDebug("getGameAttribute($p_attname) = $result");

		return $result;
	}

	function setGameAttribute ( $p_attname, $p_attvalue ) {
		global $mysql_table_game;

		$this->_att_cache = NULL;

		$id = $this->game_id;

		// If value is not a MySQL function, add quotes
		//if ( strchr($p_attvalue, '(') == NULL ) {
		//	$p_attvalue = mysql_escape_string($p_attvalue);
		//	$p_attvalue = "$p_attvalue";
		//}

		// PDO add quotes and scape the query
		$res = sql_exec("update $mysql_table_game set $p_attname=?"
				. " where gm_id=?", array($p_attvalue, $id));

		LogDebug("setGameAttribute($p_attname, $p_attvalue)");

		return $res[1]->rowCount() == 1;
	}

    function getWhiteLastMoveTimer() {
        $result = FALSE;

        if( $this->getNextPlayer() == $this->getWhitePlayer() ){
            $lastmove = $this->getLastMove();
            $result = empty($lastmove) ? -1 : $lastmove->getTimer();
        }else{
            $beforelastmove = $this->getBeforeLastMove();
            $result = empty($beforelastmove) ? -1 : $beforelastmove->getTimer();
        }

        return $result;
    }

    function getBlackLastMoveTimer() {
        $result = FALSE;

        if( $this->getNextPlayer() == $this->getBlackPlayer() ){
            $lastmove = $this->getLastMove();
            $result = empty($lastmove) ? -1 : $lastmove->getTimer();
        }else{
            $beforelastmove = $this->getBeforeLastMove();
            $result = empty($beforelastmove) ? -1 : $beforelastmove->getTimer();
        }

        return $result;
    }
};

define("GAMESET_ORDER_WHITE", "gm_player_white,gm_player_black,gm_date_start");
define("GAMESET_ORDER_BLACK", "gm_player_black,gm_player_white,gm_date_start");
define("GAMESET_ORDER_START", "gm_date_start desc,gm_player_white,gm_player_black");
define("GAMESET_ORDER_MOVES", "gm_moves desc,gm_date_start desc");
define("GAMESET_ORDER_LAST",  "gm_last_move desc,gm_date_start desc");

class GameSet {

	public $gameset_name;
	public $gameset_location;
	public $gameset_player;
	public $gameset_color;
	public $gameset_opponent;
	public $gameset_order;
	public $gameset_limit;

	function GameSet ( $p_player = NULL ) {
		$this->gameset_name = "";

		$this->gameset_player = "";
		$this->gameset_color = "";
		$this->gameset_opponent = "";
		$this->gameset_location = 'games';
		$this->gameset_order = GAMESET_ORDER_START;
		$this->gameset_limit = NULL;

		if ( $p_player ) {
			$this->gameset_player   = $p_player->getIdentifier();
			$this->gameset_opponent = "";
			$this->gameset_color    = 'anycolor';
			$this->gameset_name     = "Current games for player " . $this->gameset_player;
			$this->gameset_limit    = NULL;
			$this->gameset_location = 'games';
		}
	}

	function getGames() {
		global $mysql_table_game;
		global $mysql_table_move;
		$result = array();

		$wheres = array();
		$params = array();

		$wheres[] = "( gm_is_deleted = 0 )";

		if ( $this->gameset_player ) {
			$pl = $this->gameset_player;
			$wheres[] = "( gm_player_white=? or gm_player_black=? )";
            $params[] = $pl;
            $params[] = $pl;
		}

		if ( $this->gameset_opponent ) {
			$pl = $this->gameset_opponent;
			$wheres[] = "( gm_player_white=? or gm_player_black=? )";
            $params[] = $pl;
            $params[] = $pl;
		}

		if ( $this->gameset_color != "anycolor" ) {
			$col = $this->gameset_color;
			$pl = $this->gameset_player;

			if ( $col == "white" ) {
				$wheres[] = "( gm_player_white=? )";
                $params[] = $pl;
			}

			if ( $col == "black" ) {
				$wheres[] = "( gm_player_white=? )";
				$params[] = $pl;
			}
		}

		if ( $this->gameset_location == 'games' )  {
			$wheres[] = "( gm_is_archived=0 )";
		}
		else if ( $this->gameset_location == 'archive' )  {
			$wheres[] = "( gm_is_archived=1 )";
		}

		$where_exp = "where " . implode(" and ", $wheres);
		$query = "select gm_id, count(mv_id) as gm_moves, max(mv_date) as gm_last_move"
				. " from $mysql_table_game left join $mysql_table_move"
					. " on ( gm_id = mv_game )"
				. " $where_exp"
				. " group by gm_id"
				. " order by ?";

        $params[] = $this->gameset_order;

		if ( $this->gameset_limit ) {
			$query .= " limit ?";
			$params[] = $this->gameset_limit;
	    }

		$q = sql_query($query, $params);

		foreach($q as $row ) {
			$result[] = new Game($row['gm_id']);
		}

		return $result;
	}

	function getName() {
		return $this->gameset_name;
	}

	function setName ( $p_name ) {
		$this->gameset_name = $p_name;
	}

	function setPlayer ( $p_player ) {
		$this->gameset_player = $p_player;
	}

	function setColor ( $p_color ) {
		$this->gameset_color = $p_color;
	}

	function setOpponent ( $p_opponent ) {
		$this->gameset_opponent = $p_opponent;
	}

	function setLocation ( $p_location ) {
		$this->gameset_location = $p_location;
	}

	function setOrder ( $p_order ) {
		$this->gameset_order = $p_order;
	}

	function setLimit ( $p_limit ) {
		$this->gameset_limit = $p_limit;
	}
};

?>
