<?php

//--- Global parameters -----------------------------------------------------

$mcc_version      = array(2,3,0);
$mcc_stylesheet_common = "css/style_common.css";
$mcc_stylesheet_email  = "css/style_email.css";
$mcc_stylesheet_web    = "css/style_web.css";
$mcc_stylesheet_jqui   = "css/mkgichess-theme/jquery-ui.min.css";
$mcc_stylesheet_print  = "css/style_print.css";
$log_debug        = FALSE;

define("NOTIFICATION_ENABLED",  0);  // Emails are sent
define("NOTIFICATION_SUSPEND",  1);  // Emails are not sent, notification status is kept
define("NOTIFICATION_DISABLED", 2);  // Emails are not sent but notification flags are set
$notification_status = NOTIFICATION_ENABLED;


define("IDENTIFIER_ALLOWED_CHARS", "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_.");
define("REGEXP_EMAIL", "/^([a-zA-Z0-9]+([\.+_-][a-zA-Z0-9]+)*)@(([a-zA-Z0-9]+((\.|[-]{1,2})[a-zA-Z0-9]+)*)\.[a-zA-Z]{2,6})$/");

define("MCC_TOKEN_CHARS", "ABCDEFGHIJKLMNOP01234567890");
define("MCC_TOKEN_LENGTH", 8);

define("UNDO_DELAY_MINS",    20);
define("DROP_DELAY_MINS",    31 * 24 * 60);
define("DATEFORM_FULL",      'Y-M-d H:i');
define("DATEFORM_DATE",      'F jS');
define("NOTIFICATION_MINS",  24 * 60);
define("INVITATION_RETRY_MINS", 2 * 24 * 60);

define("HINTS_HIDING", 3);

define("CASTLE_SHORT", "O-O");
define("CASTLE_LONG",  "O-O-O");

define("RANKING_GAMES_MINI", 3);
define("RANKING_PAGE", 15);
define("RANKING_PAGE_VARNAME", "rankpage");
define("RANKING_FILTER_VARNAME", "view");
define("RANKING_FILTER_HUMANS", "humans");
define("RANKING_FILTER_ROBOTS", "robots");
define("RANKING_FILTER_ALL",    "all");

define("GAMES_PAGE", 15);
define("GAMES_PAGE_VARNAME", "glistpage");
define("GAMES_SORT_VARNAME", "glistsort");
define("GAMES_SORT_BYWHITE", "white");
define("GAMES_SORT_BYBLACK", "black");
define("GAMES_SORT_BYSTART", "start");
define("GAMES_SORT_BYMOVES", "moves");
define("GAMES_SORT_BYLAST",  "last");
define("GAMES_DEFAULT_SORT", GAMES_SORT_BYLAST);

define("SUGGESTED_PLAYERS", 15);

define("SEARCH_GAMES",   "Search games");

define("SHOW_PAGES", 10);

define("NOTIF_NEVER",     "nv");
define("NOTIF_1DAY",      "01");
define("NOTIF_2DAY",      "02");
define("NOTIF_3DAY",      "03");
define("NOTIF_4DAY",      "04");
define("NOTIF_IMMEDIATE", "im");

// These constants are ORed together to specify player list fields
define("PLAYER_IDENTIFIER",   1);
define("PLAYER_CREATIONDATE", 2);
define("PLAYER_SCOREPOINTS",  4);

// Used to check text input max size
define("MAX_TEXT_LENGTH", 2048);

require("local_settings.php");
require("mcc_robots.php");

//--- Debug tool ------------------------------------------------------------

function LogDebug ( $p_message )
{
	global $log_debug;

	if ( $log_debug ) {
		echo "\n<!-- LOG: $p_message -->";
	}
}

$querycount = 0;

function sql_get_connection($host, $db, $user, $passwd)
{
    //return new PDO("mysql:host=$host;dbname=$db;charset=utf-8", $user, $passwd,
    //    array(PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES 'utf8'"));

    return new PDO("mysql:host=$host;dbname=$db", $user, $passwd);

}

function sql_query ( $p_query, $params = NULL )
{
	global $log_debug;
	global $querycount;
	global $mysql_host, $mysql_user, $mysql_pass, $mysql_db;

	if ( $log_debug ) {
		list($usec, $sec) = explode(" ", microtime());
		$query_start = ((float)$usec + (float)$sec);
	}

    $conn = sql_get_connection($mysql_host, $mysql_db, $mysql_user, $mysql_pass);
    $stmt = $conn->prepare($p_query);
    $result = NULL;
    try{
        $stmt->execute($params);
    } catch(PDOException $e){
        LogDebug($e->getMessage());
        throw $e;
    }

	if ( $log_debug ) {
		list($usec, $sec) = explode(" ", microtime());
		$query_end = ((float)$usec + (float)$sec);
		$querycount++;

		LogDebug(
			sprintf("SQL QUERY %02d: %3dms - %s",
				$querycount,
				floor(1000 * ($query_end - $query_start)),
				$p_query));
	}

    $result = $stmt->fetchAll( PDO::FETCH_ASSOC );
	return $result;
}

/**
 * Use PDO to query database. Executes DDL queries.
 *
 * @param $p_query string the query to run
 * @param $params array the values to substitute in the placeholders of the query, the order matter!
 * @return array the pdo connection and sta used: array($conn, $stmt)
 */
function sql_exec ( $p_query, $params = NULL )
{
    global $log_debug;
    global $querycount;
    global $mysql_host, $mysql_user, $mysql_pass, $mysql_db;

    if ( $log_debug ) {
        list($usec, $sec) = explode(" ", microtime());
        $query_start = ((float)$usec + (float)$sec);
    }

    $conn = sql_get_connection($mysql_host, $mysql_db, $mysql_user, $mysql_pass);
    $stmt = $conn->prepare($p_query);

    try{
        $stmt->execute($params);
    } catch(PDOException $e){
        LogDebug($e->getMessage());
        throw $e;
    }

    if ( $log_debug ) {
        list($usec, $sec) = explode(" ", microtime());
        $query_end = ((float)$usec + (float)$sec);
        $querycount++;

        LogDebug(
            sprintf("SQL QUERY %02d: %3dms - %s",
                $querycount,
                floor(1000 * ($query_end - $query_start)),
                $p_query));
    }

    return array($conn, $stmt);
}

//--- Time management -------------------------------------------------------

function mcc_elapsed_time()
{
	global $_time_start;
	list($usec, $sec) = explode(" ", microtime());
	$now = ((float)$usec + (float)$sec);

	return floor(1000. * ( $now - $_time_start ));
}

function mcc_db_time()
{
	$q = sql_query("select UNIX_TIMESTAMP(now())");
	$res = array_shift($q);
	return $res['UNIX_TIMESTAMP(now())'];
}

//---------------------------------------------------------------------------

function mcc_get_page_parameter ( $p_name, $p_default = NULL )
{
	$value = $p_default;

	if ( isset($_GET) ) {
		if ( isset($_GET[$p_name]) ) {
			$value = $_GET[$p_name];
		}
	}

	if ( isset($_POST) ) {
		if ( isset($_POST[$p_name]) ) {
			$value = $_POST[$p_name];
		}
	}

	LogDebug("mcc_get_page_parameter(" . $p_name . ") = " . $value);

	return $value;
}

function mcc_htmlencode ( $p_text )
{
	return htmlentities($p_text, ENT_QUOTES, "UTF-8");
}

/* Makes an HTTP redirection to a given URL.
 * If the parameter is a relative URL, it will
 * first be converted to an absolute one on the
 * current chess server
 */
function mcc_http_redirect ( $p_target )
{
	global $mcc_server_root;

	// Is the target an absolute url ?
	if ( strncmp($p_target, $mcc_server_root, strlen($mcc_server_root)) == 0 ) {
		$target = $p_target;
	}
	else {
		$target = $mcc_server_root . $p_target;
	}

	header("Location: " . $target);
}

/* Check if a user is currently connected. If it is the case, his
 * Player instance is returned (new in v1.2.3). Otherwise, the session
 * is redirected to the login page.
 */
function mcc_check_login()
{
	global $maintenance_mode;

	if ( $maintenance_mode ) {
		session_destroy();
		$p = NULL;
	}
	else {
		try {
			$p = new Player();
		} catch ( Exception $e ) {
			$p = NULL;
		}
	}

	if ( ! $p ) {
		$dirs   = explode("/", $_SERVER["REQUEST_URI"]);
		$last   = array_slice($dirs, -1);
		$target = $last[0];

		mcc_http_redirect("/login.php?target=" . urlencode($target));
		exit();
	}

  $p->setPlayerLastCheck();
	return $p;
}

function mcc_check_admin_host()
{
	global $mcc_admin_host;
/*
	if ( isset($_SERVER["REMOTE_ADDR"])
			&& $mcc_admin_host != $_SERVER["REMOTE_ADDR"] ) {
		header("Location: login.php");
		exit();
	}
*/
}

function mcc_url_is_local ( $p_url )
{
	$result = TRUE;

	if ( strstr($p_url, '/') != NULL ) {
		$result = FALSE;
	}

	return $result;
}

function mcc_build_url ( $p_url, $p_varname, $p_varvalue )
{
	$uri     = $p_url;
	$varName = $p_varname;
	$varVal  = $p_varvalue;

	$result	 = '';
	$beginning = '';
	$ending	 = '';

	if (is_null($uri)) {//Piece together uri string
		$beginning = $_SERVER['PHP_SELF'];
		$ending = ( isset($_SERVER['QUERY_STRING']) ) ? $_SERVER['QUERY_STRING'] : '';
	} else {
		 $qstart = strpos($uri, '?');
		 if ($qstart === false) {
			  $beginning = $uri; //$ending is '' anyway
		 } else {
			  $beginning = substr($uri, 0, $qstart);
			  $ending = substr($uri, $qstart);
		 }
	}

	if (strlen($ending) > 0) {
		 $vals = array();
		 $ending = str_replace('?','', $ending);
		 parse_str($ending, $vals);
		 $vals[$varName] = $varVal;
		 $ending = '';
		 $count = 0;
		 foreach($vals as $k => $v) {
			  if ($count > 0) { $ending .= '&amp;'; }
			  else { $count++; }
			  $ending .= "$k=" . urlencode($v);
		 }
	} else {
		 $ending = $varName . '=' . urlencode($varVal);
	}

	$result = $beginning . '?' . $ending;

	return $result;
}

function mcc_check_email ( $p_email ) {
	$result = TRUE;

	if ( !strstr($p_email, "@") || !strstr($p_email, ".") ) {
		$result = FALSE;
	}

	$result = $result && preg_match(REGEXP_EMAIL, $p_email);

	return $result;
}

function mcc_canonical_email ( $p_email ) {
	return trim($p_email);
}

//--- Submodules ------------------------------------------------------------

require("mcc_templates.php");
require("mcc_email.php");
require("mcc_player.php");
require("mcc_chess.php");
require("mcc_note.php");
require("mcc_article.php");
require("mcc_invitation.php");
require("mcc_token.php");

//--- Sort functions --------------------------------------------------------

function sort_getOpponentsAndScores ( $a, $b )
{
	$diff_a = ($a[0] - $a[2]) / ($a[0] + $a[1] + $a[2]);
	$diff_b = ($b[0] - $b[2]) / ($b[0] + $b[1] + $b[2]);

	if ( $diff_a == $diff_b ) {
		$diff_a = $a[0] + $a[1]/2 - $a[2];
		$diff_b = $b[0] + $b[1]/2 - $b[2];
	}

	return 1000 * ($diff_b - $diff_a);
}

//--- Global initialization -------------------------------------------------

if ( ! isset($__mcc_init_done) ) {

	ini_set('display_errors', '1');
	ini_set('error_reporting', E_ALL|E_STRICT);

	date_default_timezone_set("Europe/Paris");

	list($usec, $sec) = explode(" ", microtime());
	$_time_start = ((float)$usec + (float)$sec);
	srand($_time_start);

	session_set_cookie_params(365*24*3600);
	session_start();

	//mysql_pconnect($mysql_host, $mysql_user, $mysql_pass);
	//mysql_select_db($mysql_db);

	require("handle_move.php");

	$__mcc_init_done = TRUE;
}

//##################### OCC COMPATIBILITY (deprecated API)

// I hope this will be removed soon !

$t_credits = "The chessmen graphics were taken from
<A class=\"sublink\" href=\"http://www.tommasovitale.it/my/wcg/\">
WCG</A> with kind permission of Tommaso Vitale.";
$t_main_table_width = 600;
$t_frame_color = "#eeeeff";
$t_coord_color = "#9999aa";



// These are still useful... (taken from misc.php)


/* create chess board from figure information
 * from game file and return the list of differences
 * in chessmen. positive value indicates white
 * superiority and negative values a black one.
 * list order is pawn,knight,bishop,rook,queen */
function fillChessBoard( &$board, &$w_figures, &$b_figures, $white_figs, $black_figs )
{
	$diff = array(0,0,0,0,0);

	$w_figures = explode( " ", $white_figs );
	foreach ( $w_figures as $figure )
	{
	  $type = $figure[0];
	  switch ( $type )
	  {
	    case 'P': $diff[0]++; break;
	    case 'N': $diff[1]++; break;
	    case 'B': $diff[2]++; break;
	    case 'R': $diff[3]++; break;
	    case 'Q': $diff[4]++; break;
	  }
	  $board[boardCoordToIndex(substr($figure,1))] = "w$type";
	}
	/* black figures */
	$b_figures = explode( " ", $black_figs );
	foreach ( $b_figures as $figure )
	{
	  $type = $figure[0];
	  switch ( $type )
	  {
	    case 'P': $diff[0]--; break;
	    case 'N': $diff[1]--; break;
	    case 'B': $diff[2]--; break;
	    case 'R': $diff[3]--; break;
	    case 'Q': $diff[4]--; break;
	  }
	  $board[boardCoordToIndex(substr($figure,1))] = "b$type";
	}

	if ( $diff[0] != 0 || $diff[1] != 0 || $diff[2] != 0 ||
	     $diff[3] != 0 || $diff[4] != 0 )
	  return $diff;
	else
	  return 0;
}


/* convert board coords [a-h][1-8] to 1dim index [0..63] */
function boardCoordToIndex( $coord )
{
	//echo $coord," --> ";
	switch ( $coord[0] )
	{
		case 'a': $x = 0; break;
		case 'b': $x = 1; break;
		case 'c': $x = 2; break;
		case 'd': $x = 3; break;
		case 'e': $x = 4; break;
		case 'f': $x = 5; break;
		case 'g': $x = 6; break;
		case 'h': $x = 7; break;
		default: return 64; /* erronous coord */
	}
	$y = $coord[1]-1;
	if ( $y < 0 || $y > 7 )
		return 64; /* erronous coord */
	$index = $y * 8 + $x;
	//echo "$index | ";
	return $index;
}


/* convert board index [0..63] to coords [a-h][1-8] */
function boardIndexToCoord( $index )
{
	//echo $index," --> ";
	if ( $index < 0 || $index > 63 )
		return "";
	$y = floor($index/8)+1;
	$x = chr( ($index%8)+97 );
	$coord = "$x$y";
	//echo "$coord | ";
	return $coord;
}




function getFullFigureName( $short )
{
	$name = "empty";

	switch ( $short )
	{
		case 'P': $name = "pawn"; break;
		case 'R': $name = "rook"; break;
		case 'N': $name = "knight"; break;
		case 'B': $name = "bishop"; break;
		case 'K': $name = "king"; break;
		case 'Q': $name = "queen"; break;
	}

	return $name;
}


// This was completely rewritten in MCC to use an ELO ranking

function computeExpectedScore ( $player, $opponent )
{
	return 1. / ( 1. + pow(10., ( $opponent->getScorePoints() - $player->getScorePoints() ) / 400.) );
}

function updateStats( $white, $black, $result )
{
	$pl_w = new Player($white);
	$pl_b = new Player($black);

	$user_white = array( $white,$pl_w->getScoreWins(),$pl_w->getScoreDraws(),$pl_w->getScoreLosses(),$pl_w->getScorePoints(),0,0 );
	$user_black = array( $black,$pl_b->getScoreWins(),$pl_b->getScoreDraws(),$pl_b->getScoreLosses(),$pl_b->getScorePoints(),0,0 );


	/* translate result and update both players */
	if ( $result == '-' ) {
		$score = 0.5;
		$pl_w->setScoreDraws($pl_w->getScoreDraws() + 1);
		$pl_b->setScoreDraws($pl_b->getScoreDraws() + 1);
	}
	else if ( $result == 'w' ) {
		$score = 1.0;
		$pl_w->setScoreWins($pl_w->getScoreWins() + 1);
		$pl_b->setScoreLosses($pl_b->getScoreLosses() + 1);
	}
	else {
		$score = 0.0;
		$pl_w->setScoreLosses($pl_w->getScoreLosses() + 1);
		$pl_b->setScoreWins($pl_b->getScoreWins() + 1);
	}

	$expected_w = computeExpectedScore($pl_w, $pl_b);
	$delta_w = floor(48 * ( $score - $expected_w ));
	$delta_b = -$delta_w;

	$pl_w->setScorePoints( $pl_w->getScorePoints() + $delta_w);
	$pl_b->setScorePoints( $pl_b->getScorePoints() + $delta_b);
}

?>
