<?php

class InvitationSet {
	public $iset_player;

	function InvitationSet ( $p_player ) {
		$this->iset_player = $p_player;
	}

	function getInvitations() {
		global $mysql_table_invitation;
		$result = array();

		$q = sql_query("select * from $mysql_table_invitation"
				. " where iv_player=? and iv_deleted=0"
				. " order by iv_date desc",
				array($this->iset_player->getIdentifier()));

		foreach($q as $r) {
			$result[] = new Invitation($r['iv_id']);
		}

		return $result;
	}

	function checkEmailAddress ( $p_address ) {
		global $mysql_table_invitation;
		$result = FALSE;

		$q = sql_query("select count(*) from $mysql_table_invitation"
				. " where iv_player=?"
				. "      and iv_invited_address=?",
				array($this->iset_player->getIdentifier(), $p_address));

		if ( ! empty($q) && ($r = array_shift($q)) ) {
			$result = ($r['count(*)'] == 0);
		}

		return $result;
	}
};


class Invitation {
	public $invitation_id;
	public $invitation_player;
	public $invitation_address;
	public $invitation_name;
	public $invitation_date;
	public $invitation_clicked;
	public $invitation_iplayer;
	public $invitation_deleted;

	function Invitation ( $p_id, $p_player = NULL, $p_address = NULL, $p_name = NULL ) {
		global $mysql_table_invitation;

		if ( ! $p_id && ! $p_player ) {
			throw new Exception("Must supply at least an id or a player");
		}

		if ( ! $p_id ) {
			$res = sql_exec("insert into $mysql_table_invitation"
					. " set iv_player=?,"
					. "     iv_invited_address=?,"
					. "     iv_invited_name=?,"
					. "     iv_date=now()",
					array($p_player->getIdentifier(), $p_address, $p_name));

			$p_id = $res[0]->lastInsertId();
		}

		$q = sql_query("select iv_id, iv_player, iv_invited_address,"
				. "  iv_invited_name, UNIX_TIMESTAMP(iv_date) as iv_date,"
				. "  iv_clicked, iv_opened, iv_invited_player,"
				. "  iv_retried, iv_deleted"
				. " from $mysql_table_invitation"
				. " where iv_id='$p_id'"
				. " order by iv_date desc");

		if ( ! empty($q) ) {
			$r = array_shift($q);

			if ( $r ) {
				$this->invitation_id      = $r["iv_id"];
				$this->invitation_player  = $r["iv_player"];
				$this->invitation_address = $r["iv_invited_address"];
				$this->invitation_name    = $r["iv_invited_name"];
				$this->invitation_iplayer = $r["iv_invited_player"];
				$this->invitation_date    = $r["iv_date"];
				$this->invitation_clicked = $r["iv_clicked"];
				$this->invitation_opened  = $r["iv_opened"];
				$this->invitation_retried = $r["iv_retried"];
				$this->invitation_deleted = $r["iv_deleted"];
			}
		}
	}

	function getId()      { return $this->invitation_id; }
	function getPlayer()  { return new Player($this->invitation_player); }
	function getAddress() { return $this->invitation_address; }
	function getName()    { return $this->invitation_name; }
	function getDate()    { return $this->invitation_date; }
	function isClicked()  { return $this->invitation_clicked; }
	function isJoined()   { return $this->invitation_iplayer; }
	function isOpened()   { return $this->invitation_opened; }
	function isRetried()  { return $this->invitation_retried; }
	function isDeleted()  { return $this->invitation_deleted; }

	function getInvitedPlayer() {
		$result = NULL;

		if ( $this->invitation_iplayer ) {
			try {
				$result = new Player($this->invitation_iplayer);
			} catch ( Exception $e ) {}
		}

		return $result;
	}

	function retryAllowed() {
		$result = ! $this->isRetried();

		if ( $this->invitation_iplayer ) {
			$result = FALSE;
		}

		if ( $result &&
			( time() - $this->getDate() ) < INVITATION_RETRY_MINS * 60 ) {
			$result = FALSE;
		}

		return $result;
	}

	function retry() {
		global $mysql_table_invitation;
		sql_exec("update $mysql_table_invitation"
			. " set iv_retried=1"
			. " where iv_id=?", array($this->getId()));

		return $this->invite();
	}

	function invite() {
		global $notification_status;
		$player = $this->getPlayer();

		$subject = $player->getIdentifier() . " sends you an invitation";

		$email_subject = mcc_template_email_subject($subject);
		$email_body    = mcc_template_email_invitation($this);

		$send_status = mcc_mail ( $this->getAddress(),
					  $email_subject, $email_body);

		return $send_status;
	}

	function setOpened() {
		global $mysql_table_invitation;
		sql_exec("update $mysql_table_invitation"
			. " set iv_opened=1"
			. " where iv_id=?", array($this->getId()));
	}

	function setClicked() {
		global $mysql_table_invitation;
		sql_exec("update $mysql_table_invitation"
			. " set iv_clicked=1"
			. " where iv_id=?", array($this->getId()));
	}

	function delete() {
		global $mysql_table_invitation;
		sql_exec("update $mysql_table_invitation"
			. " set iv_deleted=1"
			. " where iv_id=?", array($this->getId()));
	}

	function setInvitedPlayer ( $p_player ) {
		global $mysql_table_invitation;
		sql_exec("update $mysql_table_invitation"
			. " set iv_invited_player=?"
			. " where iv_id=?",
			array($p_player->getIdentifier(), $this->getId()));
	}
};

?>
