<?php

$_valid_playerids = array();

class Player {
	public $player_identifier;

	/* There are 3 ways to construct a Player instance:
	 *
	 * Player() => Tries to get the current session player
	 * Player(ident) => Open a given player
	 *     These 2 will fail if the player is not in the database
	 *
	 * Player(ident, pw...) => Build a new player and fill the database
	 *	  This will fail if the identifier already exists
	 *
	 * A random validation token is then generated...
	 * The player must be validated !
	 */

	function Player ( $p_identifier = NULL, $p_password = NULL,
				$p_email = NULL, $p_admin = FALSE ) {

		global $mysql_table_player;
		global $_valid_playerids;

		$admin = ($p_admin)?(1):(0);

		if ( $p_identifier && $p_password && $p_email ) {

			// We create a new player...
			$res = sql_exec("insert into $mysql_table_player set "
				. " pl_identifier=?,"
				. " pl_password=?,"
				. " pl_email_address=?,"
				. " pl_is_validated=0,"
				. " pl_is_admin=?,"
				. " pl_notification_delay=?,"
				. " pl_is_active=1,"
				. " pl_creation_date=now()", array($p_identifier, $p_password,
                    $p_email, $admin, NOTIF_1DAY));

			if ( $res[1]->rowCount() != 1 ) {
				throw new Exception("Player insertion failed");
			}

			$this->player_identifier = $p_identifier;
		}
		else {
			$identifier = NULL;

			if ( $p_identifier ) {
				$identifier = $p_identifier;
			}
			else if ( isset( $_SESSION["identifier"] ) ) {
				$identifier = $_SESSION["identifier"];
			}

			if ( $identifier == NULL ) {
				throw new Exception("Player identifier not supplied");
			}

			$this->player_identifier = $identifier;

			if ( ! in_array($identifier, $_valid_playerids) ) {
				$q = sql_query("select pl_identifier from $mysql_table_player"
						. " where pl_identifier=?"
						. "   and pl_is_validated = 1", array($identifier));

				$qres = array_shift($q);

				if ( empty($qres) ) {
					throw new Exception("Could not fetch player from database");
				}
				else {
					$this->player_identifier = $qres['pl_identifier'];
					$_valid_playerids[] = $this->player_identifier;
				}
			}
		}
	}

	function getIdentifier() {
		return $this->player_identifier;
	}

	function getPassword() {
		return $this->getPlayerAttribute('pl_password');
	}

	function setPassword ( $p_password ) {
		return $this->setPlayerAttribute('pl_password', $p_password);
	}

	function getEmailAddress() {
		return $this->getPlayerAttribute('pl_email_address');
	}

	function setEmailAddress ( $p_address ) {
		if ( mcc_check_email($p_address) ) {
			return $this->setPlayerAttribute('pl_email_address', $p_address);
		} else {
			throw new Exception("Illegal email address");
		}
	}

	function getRealName() {
		return $this->getPlayerAttribute('pl_real_name');
	}

	function setRealName ( $p_name ) {
		return $this->setPlayerAttribute('pl_real_name', $p_name);
	}

	function getCountry() {
		return $this->getPlayerAttribute('pl_country');
	}

	function setCountry ( $p_country ) {
		global $COUNTRIES;

		if ( array_key_exists($p_country, $COUNTRIES) ) {
			return $this->setPlayerAttribute('pl_country', $p_country);
		} else {
			throw new Exception("Unknown country");
		}
	}

	function getGender() {
		return $this->getPlayerAttribute('pl_gender');
	}

	function setGender ( $p_gender ) {
		global $GENDERS;

		if ( array_key_exists($p_gender, $GENDERS) ) {
			return $this->setPlayerAttribute('pl_gender', $p_gender);
		} else {
			throw new Exception("Unknown gender");
		}
	}

	function getAge() {
		return $this->getPlayerAttribute('pl_age');
	}

	function setAge ( $p_age ) {
		global $AGES;

		if ( array_key_exists($p_age, $AGES) ) {
			return $this->setPlayerAttribute('pl_age', $p_age);
		} else {
			throw new Exception("Unknown age");
		}
	}

	function getNotificationDelay() {
		return $this->getPlayerAttribute('pl_notification_delay');
	}

	function setNotificationDelay ( $p_notification_delay ) {
		global $NOTIFICATIONS;

		if ( array_key_exists($p_notification_delay, $NOTIFICATIONS) ) {
			return $this->setPlayerAttribute('pl_notification_delay', $p_notification_delay);
		} else {
			throw new Exception("Unknown notification delay");
		}
	}

	function getNewsletterSubscription() {
		return $this->getPlayerAttribute('pl_newsletter');
	}

	function setNewsletterSubscription ( $p_newsletter ) {
		return $this->setPlayerAttribute('pl_newsletter', $p_newsletter);
	}

	function getCreationDate() {
		return $this->getPlayerAttribute('unix_timestamp(pl_creation_date)');
	}

	function getActivityFlag() {
		return $this->getPlayerAttribute('pl_is_active');
	}

	function setActivityFlag ( $p_flag ) {
		return $this->setPlayerAttribute('pl_is_active', $p_flag);
	}

	function isValidated() {
		return $this->getPlayerAttribute('pl_is_validated');
	}

	function validate() {
		return $this->setPlayerAttribute('pl_is_validated', 1);
	}

	function isRobot() {
		return $this->getPlayerAttribute('pl_email_address') == "";
	}

	function isAdmin() {
		return $this->getPlayerAttribute('pl_is_admin') == 1;
	}

	function registerSession() {
		global $mysql_table_h_connection;

		$_SESSION["identifier"] = $this->player_identifier;

		if ( ! $this->isRobot() ) {
			sql_exec("insert into $mysql_table_h_connection set "
					. " hc_identifier=?,"
					. " hc_date=now()",
					array($this->getIdentifier()));
		}
	}

	function getScoreWins() {
		return $this->getPlayerAttribute('pl_score_wins');
	}

	function setScoreWins ( $p_value ) {
		return $this->setPlayerAttribute('pl_score_wins', $p_value);
	}

	function getScoreDraws() {
		return $this->getPlayerAttribute('pl_score_draws');
	}

	function setScoreDraws ( $p_value ) {
		return $this->setPlayerAttribute('pl_score_draws', $p_value);
	}

	function getScoreLosses() {
		return $this->getPlayerAttribute('pl_score_losses');
	}

	function setScoreLosses ( $p_value ) {
		return $this->setPlayerAttribute('pl_score_losses', $p_value);
	}

	function getScoreGames() {
		return $this->getPlayerAttribute('pl_score_wins + pl_score_draws + pl_score_losses');
	}

	function getScorePoints() {
		return $this->getPlayerAttribute('pl_score_points');
	}

	function setScorePoints ( $p_value ) {
		global $mysql_table_h_score;

		sql_exec("insert into $mysql_table_h_score set "
				. " hs_identifier=?,"
				. " hs_date=now(), hs_score_points=?",
                array($this->getIdentifier(), $p_value));

		return $this->setPlayerAttribute('pl_score_points', $p_value);
	}

    function setPlayerLastCheck ( ) {
        global $mysql_table_player;

        sql_exec("update $mysql_table_player set "
          . "pl_last_check = now() "
          . "where pl_identifier = ? ", array($this->player_identifier));
    }

    function getPlayerLastCheck ( ) {
        return $this->getPlayerAttribute('unix_timestamp(pl_last_check)');
    }

	// --- Internal API

	function getPlayerAttribute ( $p_attname ) {
		global $mysql_table_player;
		$result = NULL;

		$id = $this->player_identifier;

		$q = sql_query("select $p_attname from $mysql_table_player"
				. " where pl_identifier=?", array($id));

		if ( ! empty($q) ) {
			$row = array_shift($q);
			$result = $row[$p_attname];
		}

		return $result;
	}

	function setPlayerAttribute ( $p_attname, $p_attvalue ) {
		global $mysql_table_player;
		$id = $this->player_identifier;

		if ( $p_attvalue === NULL ) {
			$p_attvalue = "NULL";
		}
		else {
			$p_attvalue = "'" . $p_attvalue . "'";
		}

		$res = sql_exec("update $mysql_table_player set $p_attname=$p_attvalue"
				. " where pl_identifier=?", array($id));

		return $res[1]->rowCount() == 1;
	}

	//-- Portrait -----------------------

	function getPortraitType() {
		global $mysql_table_player_portrait;
		$id = $this->player_identifier;

		$q = sql_query("select pp_type from $mysql_table_player_portrait"
				. " where pp_player=?", array($id));

		if ( $q ) {
			$row = array_shift($q);
			$result = $row['pp_type'];
		}

		return $result;
	}

	function getPortraitSize() {
		global $mysql_table_player_portrait;
		$id = $this->player_identifier;

		$q = sql_query("select length(pp_data) from $mysql_table_player_portrait"
				. " where pp_player=?", array($id));

        $result = null;

		if ( ! empty($q) ) {
			$row = array_shift($q);
			$result = $row['length(pp_data)'];
		}

		return $result;
	}

	function getPortraitData() {
		global $mysql_table_player_portrait;
		$id = $this->player_identifier;

		$q = sql_query("select pp_data from $mysql_table_player_portrait"
				. " where pp_player=?", array($id));

		if ( ! empty($q) ) {
			$row = array_shift($q);
			$result = $row['pp_data'];
		}

		return $result;
	}

	function setPortrait ( $p_type, $p_data ) {
		global $mysql_table_player_portrait;
		$id = $this->player_identifier;

		sql_exec("replace $mysql_table_player_portrait set "
			     . " pp_player=?, pp_type=?,"
			     . " pp_data=?", array($id, $p_type, $p_data));
	}

	//-- History ------------------------

	function getScoreHistory() {
		global $mysql_table_h_score;
		$id = $this->player_identifier;

		$q = sql_query("select unix_timestamp(hs_date), hs_score_points"
			        . " from $mysql_table_h_score"
				. " where hs_identifier='$id'"
				. " order by hs_date");

		$result = array();

		foreach( $q as $row ) {
			$result[$row['unix_timestamp(hs_date)']] = $row['hs_score_points'];
		}

		return $result;
	}

	//-- Profile data -------------------

	function getOpponentsAndScores() {
		global $mysql_table_game;
		$id = $this->player_identifier;

		$result = array();

		$q = sql_query("select gm_player_white, gm_player_black, gm_status"
				. " from $mysql_table_game"
				. " where (gm_player_white='$id' or gm_player_black='$id')"
				. "       and gm_status != 'open'");

		foreach($q as $row) {
			// I was white...
			if ( $id == $row['gm_player_white'] ) {
				$opponent = $row['gm_player_black'];
			}
			// I was black...
			else {
				$opponent = $row['gm_player_white'];
			}

			if ( isset($result[$opponent]) ) {
				$score = $result[$opponent];
			}
			else {
				$score = array(0,0,0);
			}

			if ( $row['gm_status'] == 'draw' ) {
				$score[1]++;
			}
			else if (  $row['gm_player_white'] == $id && $row['gm_status'] == 'whitewon'
				or $row['gm_player_black'] == $id && $row['gm_status'] == 'blackwon' ) {

				$score[0]++;
			}
			else {
				$score[2]++;
			}

			$result[$opponent] = $score;
		}

		uasort($result, 'sort_getOpponentsAndScores');

		return $result;
	}

	//-- Emails -------------------------

	function sendSubscriptionResult ( $p_again = FALSE ) {
		$subject = "Subscription";

		if ( $p_again ) {
			$subject .= " (reminder)";
		}

		$token = new EmailToken($this->player_identifier,
					NULL, $this->getEmailAddress());

		$email_subject = mcc_template_email_subject($subject);
		$email_body    = mcc_template_email_subscription($this, $token);
		$send_status = mcc_mail ( $this->getEmailAddress(),
					  $email_subject, $email_body);

		return $send_status;
	}

	function sendLostPassword() {
		$email_subject = mcc_template_email_subject("Lost Password");
		$email_body    = mcc_template_email_lostpassword($this);
		$send_status = mcc_mail ( $this->getEmailAddress(),
					  $email_subject, $email_body);

		return $send_status;
	}

	function sendEmailValidation ( $p_emailtoken ) {
		$email_subject = mcc_template_email_subject("Confirm email address");
		$email_body    = mcc_template_email_changeaddress($this, $p_emailtoken);
		$send_status = mcc_mail ( $p_emailtoken->getEmailAddress(),
					  $email_subject, $email_body);
		return $send_status;
	}

	function sendMoveNotification ( $p_game, $p_dropsoon = false ) {
		global $notification_status;

		$opponent = $p_game->getOpponent($this);

		if ( $p_game->getMoveCount() >= 2 ) {
			$subject = "Your game against " . $opponent->getIdentifier();
		}
		else {
			$subject = "New game started against " . $opponent->getIdentifier();
		}

		if ( $p_dropsoon ) {
			$subject .= " (Warning !)";
		}

		$email_subject = mcc_template_email_subject($subject);
		$email_body    = mcc_template_email_move_notification($this, $p_game, $p_dropsoon);

		if ( $notification_status == NOTIFICATION_ENABLED ) {
			$send_status = mcc_mail ( $this->getEmailAddress(),
						  $email_subject, $email_body);
		}
		else {
			echo "*** Mail '$email_subject' not sent to " . $this->getEmailAddress() . "\n";
			$send_status = TRUE;
		}

		if ( $send_status && $notification_status != NOTIFICATION_SUSPEND ) {
			$p_game->setLastnotificationDate();
		}

		return $send_status;
	}

	//-- Robots -------------------------

	function getDepth() {
		$depth = NULL;

		if ( $this->isRobot() ) {
			$ending = strrchr($this->player_identifier, '_');
			$depth_str = substr($ending, 1);
			$depth = intval($depth_str);
		}

		return $depth;
	}

	function computeNextMove ( $p_game ) {
		if ( strpos($this->player_identifier, "gnuchess") != false ) {
			$move = ComputeNextMove_Gnuchess($p_game->getMoves(), $this->getDepth());
		} else if ( strpos($this->player_identifier, "phalanx") != false ) {
			$move = ComputeNextMove_Phalanx($p_game->getMoves(), $this->getDepth());
		}

		return $move;
	}
};

class UnvalidatedPlayer extends Player {
	/* An unvalidated player is a temporary state between subscription
	 * and validation. We must define this specific constructor in
	 * order to be able to create a Player object
	 */
	function UnvalidatedPlayer ( $p_identifier = NULL ) {
		global $mysql_table_player;
		$this->player_identifier = $p_identifier;

		$q = sql_query("select pl_identifier from $mysql_table_player"
				. " where pl_identifier='$p_identifier'"
				. "   and pl_is_validated = 0");

		if ( empty($q) ) {
			throw new Exception("Could not fetch player from database");
		}
	}

	function delete () {
		global $mysql_table_player;
		$id = $this->player_identifier;
		$q = sql_exec("delete from $mysql_table_player where pl_identifier='$id'");
		return $q[1]->rowCount() == 1;
	}
};


class PlayerSet {

	function getIdentifiers() {
		global $mysql_table_player;
		$result = array();

		$q = sql_query("select pl_identifier from $mysql_table_player where pl_is_validated = 1 order by pl_identifier");

		foreach($q as $row) {
			$result[] = $row['pl_identifier'];
		}

		return $result;
	}

	function getUnvalidatedIdentifiers() {
		global $mysql_table_player;
		$result = array();

		$q = sql_query("select pl_identifier from $mysql_table_player where pl_is_validated = 0 order by pl_identifier");

		foreach($q as $row) {
			$result[] = $row['pl_identifier'];
		}

		return $result;
	}

	function getPlayersByRankDesc ( $p_mingames = 0, $p_filter ) {
		global $mysql_table_player;
		global $mysql_table_game;

		$result = array();

		if ( $p_filter == RANKING_FILTER_HUMANS ) {
			$where = " and pl_email_address is not NULL";
		}
		else if ( $p_filter == RANKING_FILTER_ROBOTS ) {
			$where = " and pl_email_address is NULL";
		}
		else {
			$where = "";
		}

		$q = sql_query("select pl_identifier, count(gm_id)"
				. " from $mysql_table_player left join $mysql_table_game on"
				. " ( gm_player_white = pl_identifier or gm_player_black=pl_identifier )"
				. " where pl_is_active = 1 and gm_status != 'open'" . $where
				. " group by pl_identifier"
				. " order by pl_score_points desc");


		foreach($q as $row) {
			$p = new Player($row['pl_identifier']);

			if ( $p && $row['count(gm_id)'] >= $p_mingames ) {
				$result[] = $p;
			}
		}

		return $result;
	}

	function getNewbies ( $p_nbplayers = SUGGESTED_PLAYERS ) {
		global $mysql_table_player;
		$result = array();

		$q = sql_query("select pl_identifier"
				. " from $mysql_table_player"
				. " where pl_is_active = 1"
				. "       and pl_is_validated = 1"
				. "       and pl_email_address is not null"
				. " order by pl_creation_date desc"
				. " limit $p_nbplayers");

		foreach($q as $row) {
			$result[] = new Player($row['pl_identifier']);
		}

		return $result;
	}

	function getSuggestedOpponents ( $p_player, $p_nbplayers = SUGGESTED_PLAYERS ) {
		global $mysql_table_player;
		global $mysql_table_game;

		$result = array();

		$q = sql_query("select pl_identifier, abs(" . $p_player->getScorePoints() . " - pl_score_points) * (count(gm_id)+1)"
				. " from $mysql_table_player"
				. "  left join $mysql_table_game on"
					. " ( pl_identifier in (gm_player_black, gm_player_white)"
					. "   and '" . $p_player->getIdentifier() . "' in (gm_player_black, gm_player_white))"
				. " where pl_is_active = 1 and pl_is_validated = 1 and pl_email_address is not null"
					. " and pl_identifier != '" . $p_player->getIdentifier() . "'"
				. " group by pl_identifier"
				. " order by 2 asc"
				. " limit $p_nbplayers");


		foreach($q as $row) {
			$result[] = new Player($row['pl_identifier']);
		}

		return $result;
	}

	function getOnlinePlayers() {
        global $mysql_table_player, $mcc_ajax_interval_user;

        $result = array();

        $q = sql_query("select pl_identifier, pl_score_points"
            . " from $mysql_table_player"
            . " where pl_is_active = 1"
            . "       and pl_is_validated = 1"
            . "       and pl_email_address is not null"
            . "       and ( unix_timestamp( now() ) - unix_timestamp(pl_last_check) ) <= $mcc_ajax_interval_user / 1000"
            . " order by pl_identifier desc");

        foreach($q as $row) {
          $result[$row['pl_identifier']] = $row['pl_score_points'];
        }

        return $result;
    }

    function getAllPlayersByRankDesc ( $p_mingames = 0, $p_filter ) {
        global $mysql_table_player;
        global $mysql_table_game;

        $result = array();

        if ( $p_filter == RANKING_FILTER_HUMANS ) {
            $where = " and pl_email_address is not NULL";
        }
        else if ( $p_filter == RANKING_FILTER_ROBOTS ) {
            $where = " and pl_email_address is NULL";
        }
        else {
            $where = "";
        }

        $q = sql_query("select pl_identifier, count(gm_id)"
                . " from $mysql_table_player left join $mysql_table_game on"
                . " ( gm_player_white = pl_identifier or gm_player_black=pl_identifier )"
                . " where pl_is_active = 1 and pl_is_validated = 1 " . $where
                . " group by pl_identifier"
                . " order by pl_score_points desc");


        foreach($q as $row) {
            $p = new Player($row['pl_identifier']);

            if ( $p && $row['count(gm_id)'] >= $p_mingames ) {
                $result[] = $p;
            }
        }

        return $result;
    }

};

?>
