<?php

//--- Page Components -------------------------------------------------------

function mcc_template_box ( $p_title, $p_body, $p_width = 600 )
{
	if ( $p_width ) {
		$style = "style='width: ${p_width}px;'";
	}
	else {
		$style = "";
	}

	return <<<EOT
	<table class="mccbox" $style>
	<tr><td class="mccbox_title">
		$p_title
	</td></tr>
	<tr><td class="mccbox_body">
		$p_body
	</td></tr>
	</table> <!-- mccbox end -->
EOT;
}

function mcc_template_sidebox ( $p_title, $p_body )
{
	return <<<EOT
	<table class="mccbox" style="width: 200px;">
	<tr><td class="mccbox_title" style="font-size: 8pt;">
		$p_title
	</td></tr>
	<tr><td class="mccbox_body" style="font-size: 8pt;">
		$p_body
	</td></tr>
	</table> <!-- mccbox end -->
EOT;
}


function mcc_template_suggestions ( $p_player )
{
	$playerset = new PlayerSet();
	$newbies = $playerset->getNewbies();
	$suggest = $playerset->getSuggestedOpponents($p_player);

	$newbies_html = mcc_template_playerlist ( $newbies,
				PLAYER_IDENTIFIER | PLAYER_CREATIONDATE,
				"New Members",
				"These members joined the club recently"
				. " and we suggest that you invite them for"
				. " their first games."
				);

	$suggest_html = mcc_template_playerlist ( $suggest,
				PLAYER_IDENTIFIER | PLAYER_SCOREPOINTS,
				"Members you could play against",
				"They have been selected because we think that"
				. " they could be an interresting match for you."
				);

	$result = "";
	$result .= "<table><tr>\n";
	$result .= "<td style='vertical-align: top; padding-right: 10px;'>$newbies_html</td>\n";
	$result .= "<td style='vertical-align: top;'>$suggest_html</td>\n";
	$result .= "</tr></table>\n";

	$result = mcc_template_box("Suggested Opponents", $result);
	return $result;
}

function mcc_template_playerlist ( $p_players, $p_fields, $p_title, $p_abstract )
{
	$result = "";

	$nbfields = 0;
	if ( $p_fields & PLAYER_IDENTIFIER ) { $nbfields++; }
	if ( $p_fields & PLAYER_CREATIONDATE ) { $nbfields++; }
	if ( $p_fields & PLAYER_SCOREPOINTS ) { $nbfields++; }


	$result .= "<table class='playerlist'>";
	$result .= "<tr>";
	$result .= "<td class='playerlist_title' colspan='$nbfields'>$p_title</td>";
	$result .= "</tr>";
	$result .= "";

	foreach ( $p_players as $player ) {
		$result .= "<tr>";

		if ( $p_fields & PLAYER_IDENTIFIER ) {
			$id = $player->getIdentifier();
			$result .= "<td class='playerlist'>"
				. "<a href='player_profile_view.php?player=$id'>"
				. "$id</a></td>";
		}

		if ( $p_fields & PLAYER_SCOREPOINTS ) {
			$result .= "<td class='playerlist' style='text-align: right;'>"
				. $player->getScorePoints() . "</td>";
		}

		if ( $p_fields & PLAYER_CREATIONDATE ) {
			$result .= "<td class='playerlist'>"
				. date(DATEFORM_DATE, $player->getCreationDate()) . "</td>";
		}

		$result .= "";
		$result .= "</tr>";
	}

	$result .= "<tr>";
	$result .= "<td class='playerlist_abstract' colspan='$nbfields'>$p_abstract</td>";
	$result .= "</tr>";
	$result .= "</table>\n";

	return $result;
}

function mcc_template_news ( $p_articleset )
{
	global $mcc_server_name;
	$result = "";

	$art_count = $p_articleset->size();

	if ( $art_count > 0 ) {
		$newsbody = "<dl class=\"news\">";

		for ( $art = 0; $art < $art_count; $art++ ) {
			$article = $p_articleset->getArticle($art);

			$title  = date(DATEFORM_DATE,$article->getDate());
			$title .= " - " . $article->getTitle();
			$text   = $article->getText();
			$text  .= "<p class=\"news_signature\">"
				. $article->getAuthor() . "</p>\n";

			$newsbody .= "<dt class=\"news\">$title</dt>\n";
			$newsbody .= "<dd class=\"news\">$text</dd>\n";
		}

		$newsbody .= "</dl>";

		$result = mcc_template_box($mcc_server_name . " News", $newsbody);
	}

	return $result;
}


function mcc_template_tip ( $p_tip )
{
	$result = "";
	$text = $p_tip->getText();

	if ( $text ) {
		$result = mcc_template_sidebox("Tip", $text);
	}

	return $result;
}

function mcc_template_userinfos ( $p_player )
{
	global $AGES, $GENDERS, $COUNTRIES;

	$pp_identifier  = $p_player->getIdentifier();
	$pp_realname    = $p_player->getRealName();
	$pp_country     = $p_player->getCountry();
	$pp_gender      = $p_player->getGender();
	$pp_age         = $p_player->getAge();
	$pp_points      = $p_player->getScorePoints();
	$pp_games       = $p_player->getScoreGames();
	$pp_wins        = $p_player->getScoreWins();
	$pp_draws       = $p_player->getScoreDraws();
	$pp_losses      = $p_player->getScoreLosses();

	$pp_realname = mcc_htmlencode($pp_realname);

	// Player data processing

	if ( $pp_age ) {
		$pp_age = $AGES[$pp_age];
	}

	if ( $pp_gender ) {
		$pp_gender = $GENDERS[$pp_gender];
	}

	if ( $pp_country ) {
		$pp_country = $COUNTRIES[$pp_country];
	}

	return <<<EOT
	<table class="profile">
	<tr>
	<td class="profile_label">Real Name</td>  <td class="profile" colspan="3">$pp_realname</td>
	</tr>
	<tr>
	<td class="profile_label">Elo</td>        <td class="profile">$pp_points</td>
	<td class="profile_label">Games</td>      <td class="profile" style="text-align: right;">$pp_games</td>
	</tr>
	<tr>
	<td class="profile_label">Gender</td>     <td class="profile">$pp_gender</td>
	<td class="profile_label">Wins</td>       <td class="profile" style="text-align: right;">$pp_wins</td>
	</tr>
	<tr>
	<td class="profile_label">Country</td>    <td class="profile">$pp_country</td>
	<td class="profile_label">Draws</td>      <td class="profile" style="text-align: right;">$pp_draws</td>
	</tr>
	<tr>
	<td class="profile_label">Age</td>        <td class="profile">$pp_age</td>
	<td class="profile_label">Losses</td>     <td class="profile" style="text-align: right;">$pp_losses</td>
	</tr>
	</table>
EOT;
}

function mcc_template_online_players ( $online_players, $ajax = FALSE )
{
    global $mcc_server_root;

    $result = ""; $list = "";

    foreach ($online_players as $id => $elo) {
      $pl_url = $mcc_server_root . "/player_profile_view.php?player=" . $id;
      $list .= "<li><a href=\"$pl_url\">$id</a> - $elo</li>";
    }

    if ( $online_players && ! $ajax ) {
      $text = '<ul id="online-players">';
      $text .= $list;
      $text .= '</ul>';
      $result = mcc_template_sidebox("Players Online", $text);
      return $result;
    }

    // else return only the list
    return $list;
}

?>