<?php

function mcc_template_email_subject ( $p_text ) {
	global $mcc_server_name;

	return "[$mcc_server_name] $p_text";
}


function mcc_template_email_subscription ( $p_player, $p_token )
{
	global $mcc_server_name;
	global $mcc_server_root;

	$id = $p_player->getIdentifier();
	$pw = $p_player->getPassword();
	$va = $p_token->getValue();

	$val_url = "{$mcc_server_root}player_validation.php?validation={$va}";

	$contents = <<<EOT
<p>You have been successfully suscribed to $mcc_server_name.</p>
<p>Your username is &nbsp;<strong>$id</strong>&nbsp; and your password
is &nbsp;<strong>$pw</strong>.</p>

<p>The last step required to confirm your subscription is to click on
the link below, or to copy/paste this address to your web browser...</p>

<p><a href="{$val_url}">{$val_url}</a></p>
EOT;

	return mcc_template_email($p_player, $contents);
}


function mcc_template_email_changeaddress ( $p_player, $p_token )
{
	global $mcc_server_name;
	global $mcc_server_root;

	$id = $p_player->getIdentifier();
	$pw = $p_player->getPassword();
	$va = $p_token->getValue();

	$val_url = "{$mcc_server_root}player_validation.php?validation={$va}";

	$contents = <<<EOT
<p>You have asked to replace your current email address on $mcc_server_name.</p>

<p>You still need to confirm this email address by clicking on
	the link below, or by copy/pasting this address to your web browser...</p>

<p><a href="{$val_url}">{$val_url}</a></p>
EOT;

	return mcc_template_email($p_player, $contents);
}


function mcc_template_email_lostpassword ( $p_player )
{
	global $mcc_server_name;
	global $mcc_server_root;

	$id = $p_player->getIdentifier();
	$pw = $p_player->getPassword();

	$contents = <<<EOT

<p>We send you your $mcc_server_name login informations as someone
   requested it.</p>

<p>Your username : &nbsp; <strong>$id</strong> <br />
<p>Your password : &nbsp; <strong>$pw</strong> <br />

<p>Now you can go to <a href="$mcc_server_root">$mcc_server_root</a> and
	log in to play.</p>

EOT;

	return mcc_template_email($p_player, $contents);
}

function mcc_template_email_invitation ( $p_invitation )
{
	global $mcc_server_root;
	global $mcc_server_name;
	global $mcc_server_email;

	$tickerurl = $mcc_server_root . "mail_ticker.php?invitation=" . $p_invitation->getId();
	$joinurl   = $mcc_server_root . "player_subscription.php?invitation=" . $p_invitation->getId();

	$contents = <<<EOT
	<p style="text-align: center;"><a href="$joinurl"><img src="$tickerurl" alt="Site Logo" /></a></p>
	<p>Hello fellow chess enthusiast,</p>
	<p>$mcc_server_name is a Chess website where users play aginst each other and against chess engines.
		What makes it special is that it doesn't take much time to play since games are always
		saved and everyone can play a few moves, even if you don't have a long time to spend.
	</p>
	<p>You'll have a detailed personal profile which will allow you to be present in our permanent ladder.</p>

	<p>Join today, it's 100% free and fun !</p>
	<p><a href="$joinurl">Click here to join $mcc_server_name</a></p>
EOT;
	return mcc_template_email(NULL, $contents);
}


function mcc_template_email_move_notification ( $p_player, $p_game, $p_dropsoon )
{
	global $mcc_server_root;
	global $mcc_server_name;
	global $mcc_server_email;

	$ac_move = "";
	$w_figures = array();
	$b_figures = array();
	$attackers = array();
	$board = array( 
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  ",
		"  ", "  ", "  ", "  ", "  ", "  ", "  ", "  " );

	$game = $p_game->initializeLegacyVariables();
	$diff = fillChessBoard($board, $w_figures, $b_figures, trim($game[2]), trim($game[3]) );


	$html_board = mcc_template_board($p_game, $p_player == $p_game->getBlackPlayer(),
					BOARDVIEW_STANDARD, FALSE, $board, $diff);

	$html_history  = "<div style=\"border: 1px dotted #A0A0B0; padding: 7px;\">\n";
	$html_history .= mcc_template_move_history($p_game);
	$html_history .= "</div>\n";

	$player_games = new GameSet($p_player);

	$html_gameslist = "";

	if ( count($player_games->getGames()) > 1 ) {
		$player_games->setName("Your current games on $mcc_server_name");
		$html_gameslist  = '<div style="margin-top: 40px;">';
		$html_gameslist .= mcc_template_gameslist($player_games, $p_player);
		$html_gameslist .= "</div>\n";
	}

	$opponent = $p_game->getOpponent($p_player);
	$oppname  = $opponent->getIdentifier();

	$lastmove = $p_game->getLastMove();
	if ( $lastmove ) {
		$movedesc = $lastmove->getLong();
		$chatter  = $lastmove->getChatter();
	}
	else {
		$movedesc = "";
		$chatter  = "";
	}

	$gameurl  = $mcc_server_root . "chess.php?gameid=" . $p_game->getId();

	if ( $lastmove ) {
		$movetext = "<p>Your opponent &nbsp; <strong>$oppname</strong>"
				. " &nbsp; played &nbsp; <strong>$movedesc</strong>.</p>";
	}
	else {
		$movetext = "<p style=\"font-style: italic;\">No moves yet...</p>";
	}

	if ( $chatter != "" ) {
		$movechat  = "";
		$movechat .= "<div style=\"background: #F2F2F2; padding: 10px; width: 550px; \">";
		$movechat .= "<p style=\"border-bottom: 1px solid #A0A0B0; margin: 0 0 4px 0;\">Chatter message from $oppname:</p>";
		$movechat .= "<p style=\"margin: 4px 0 0 0; font-style: italic; font-family: monospace;\">";
		$movechat .= str_replace("\n", "<br />", $chatter);
		$movechat .= "</p>\n";
		$movechat .= "</div>\n";
	}
	else {
		$movechat  = "<p><cite>No chatter from $oppname</cite></p>\n";
	}

	if ( $p_dropsoon ) {
		$dropwarning  = "<p class=\"warning\">";
		$dropwarning .= "Warning: If you don't play your move soon, your opponent will be able to cancel";
		$dropwarning .= " this game and win !";
		$dropwarning .= "</p>\n";
	}
	else {
		$dropwarning = "";
	}

	$contents = <<<EOT

<table summary="Chess Game Informations"> <tr> 

<td colspan="2">
	$dropwarning
	$movetext
	$movechat
</td>

</tr> <tr> <!-- -->

<td style="padding: 10px;">
	$html_board
</td>
<td style="padding: 10px 10px 10px 40px;">
	$html_history
</td>

</tr> <tr> <!-- -->

<td colspan="2">
	<p style="text-align: center; margin-top: 16px;">
	[ <a href="$gameurl">Click here to open this game on $mcc_server_name</a> ]
	</p>
</td>

</tr> <tr> <!-- -->

<td colspan="2">
	$html_gameslist
</td>

</tr> </table> <!-- -->

<p style="font-size: smaller;">
If you want to change your email notification settings, you
can do so when logged on the site.<br />
Just click <cite>My Profile / Change Your Email Settings </cite>.
You'll then be able to update your email address and/or change
the notification delay.
</p>

EOT;

	return mcc_template_email($p_player, $contents);
}


function mcc_template_email ( $p_player, $p_contents )
{
	global $mcc_stylesheet_common;
	global $mcc_stylesheet_email;
	global $mcc_version;
	global $mcc_server_name;

	$version = "v" . implode(".", $mcc_version);

	$email  = "";

	$email .= <<<EOT
	<html>
	<head>
	<meta http-equiv='content-type' content='text/html; charset=utf-8' />
	<style type="text/css">
EOT;

	$email .= implode("", file($mcc_stylesheet_common));
	$email .= implode("", file($mcc_stylesheet_email));

	$email .= <<<EOT
	</style>
	</head>
	<body>
EOT;

	if ( $p_player ) {
		$email .= mcc_template_userbar($p_player, "User &nbsp;");
	}

	$year = date('Y');

	$email .= "<div style=\"padding: 13px 15px 10px 15px;\">";
	$email .= $p_contents;

	$email .= <<<EOT

	<p>&nbsp;</p>
	<p>Have fun !</p>

	<p>The $mcc_server_name Team</p>

	</div>

	<table style="width: 100%; border-top: 1px solid #F2F2F2;"
		summary="Signature">
	<tr>
		<td class="tiny" style="vertical-align: top; text-align: left; padding: 3px;">
		  MKGI Chess Club $version &nbsp;
		  &copy; $year
		  <a class="sublink" href="http://www.alcibiade.org/">Alcibiade.org</a>
		</td>
		<td class="tiny" style="vertical-align: top; text-align: right;">
		</td>
	</tr>
	</table>
	</body></html>
EOT;

	return $email;
}

?>