<?php

// Warning: We only manage condensed lists without sortlinks

function mcc_template_gameslist ( $p_gameset, $p_current_player = NULL, $p_sortlinks = FALSE,
					$p_absolute = TRUE, $p_paging = TRUE, $p_condensed = FALSE )
{
	global $mcc_server_root;

	if ( $p_absolute ) {
		$imgroot = $mcc_server_root . "/images";
	}
	else {
		$imgroot = "images";
	}

	$sortkey = mcc_get_page_parameter(GAMES_SORT_VARNAME);

	if ( $sortkey ) {
		switch ( $sortkey ) {
		case GAMES_SORT_BYWHITE: $p_gameset->setOrder(GAMESET_ORDER_WHITE);	break;
		case GAMES_SORT_BYBLACK: $p_gameset->setOrder(GAMESET_ORDER_BLACK);	break;
		case GAMES_SORT_BYSTART: $p_gameset->setOrder(GAMESET_ORDER_START);	break;
		case GAMES_SORT_BYMOVES: $p_gameset->setOrder(GAMESET_ORDER_MOVES);	break;
		case GAMES_SORT_BYLAST:  $p_gameset->setOrder(GAMESET_ORDER_LAST);	break;
		}
	}

	$result = "";

	$games = $p_gameset->getGames();

	if ( count($games) < 2 ) {
		$p_sortlinks = FALSE;
	}

	$total_pages = ceil(count($games) / GAMES_PAGE);

	$gpage = mcc_get_page_parameter(GAMES_PAGE_VARNAME);

	if ( $gpage ) {
		if ( $gpage < 1 ) {
			$gpage = 1;
		}

		if ( $gpage > $total_pages ) {
			$gpage = $total_pages;
		}
	}
	else {
		$gpage = 1;
	}

	$gmlbl   = "game" . ((count($games)==1)?"":"s");
	$result .= "<p class=\"games_title\">";
	$result .= $p_gameset->getName() . " (" . count($games). " $gmlbl)";
	$result .= "</p>";

	if ( count($games) == 0 ) {
		$result .= "<p style=\"text-align: center;\">No games.</p>";

		if ( ! $p_condensed ) {
			$result .= "<p style=\"text-align: center; font-style: italic;\">You can"
				. " look for opponents in the <a href=\"rankings.php\">rankings section</a>.</p>";
		}
	}
	else {
		$surl_w = mcc_build_url(NULL, GAMES_SORT_VARNAME, GAMES_SORT_BYWHITE);
		$surl_b = mcc_build_url(NULL, GAMES_SORT_VARNAME, GAMES_SORT_BYBLACK);
		$surl_s = mcc_build_url(NULL, GAMES_SORT_VARNAME, GAMES_SORT_BYSTART);
		$surl_m = mcc_build_url(NULL, GAMES_SORT_VARNAME, GAMES_SORT_BYMOVES);
		$surl_l = mcc_build_url(NULL, GAMES_SORT_VARNAME, GAMES_SORT_BYLAST);

		if ( $p_sortlinks ) {
			$result .= <<<EOT
			<table class="games">
			<tr>
			<td class="games_title"></td>
			<td class="games_title"><a class="games_title" href="$surl_w">White Player</a></td>
			<td class="games_title"><a class="games_title" href="$surl_b">Black Player</a></td>
			<td class="games_title"><a class="games_title" href="$surl_s">Starting Date</a></td>
			<td class="games_title"><a class="games_title" href="$surl_m">Moves</a></td>
			<td class="games_title"><a class="games_title" href="$surl_l">Last Move At</a></td>
			<td class="games_title"></td>
			</tr>
EOT;
		}
		else {
			if ( $p_condensed ) {
				$result .= <<<EOT
				<table class="games" style="width: 300px;">
				<tr>
				<td class="games_title">White</td>
				<td class="games_title">Black</td>
				<td class="games_title">Moves</td>
				<td class="games_title"></td>
				</tr>
EOT;
			}
			else {
				$result .= <<<EOT
				<table class="games">
				<tr>
				<td class="games_title"></td>
				<td class="games_title">White Player</td>
				<td class="games_title">Black Player</td>
				<td class="games_title">Starting Date</td>
				<td class="games_title">Moves</td>
				<td class="games_title">Last Move At</td>
				<td class="games_title"></td>
				</tr>
EOT;
			}
		}

		if ( $p_paging ) {
			$pagegames = array_slice($games, ($gpage - 1) * GAMES_PAGE, GAMES_PAGE);
		}
		else {
			$pagegames = $games;
		}

		foreach ( $pagegames as $game ) {
			$myturn = ! $game->isArchived() && $p_current_player == $game->getNextPlayer();

			if ( $myturn ) {
				$light = "lamp_green.png";
			}
			else {
				$light = "lamp_red.png";
			}

			if ( $game->isArchived() ) {
				$script = "/browser.php";
			}
			else {
				$script = "/chess.php";
			}

			if ( $p_absolute ) {
				$script = $mcc_server_root . $script;
			}

			$white = $game->getWhitePlayer();
			$black = $game->getBlackPlayer();

			if ( $game->getWinner() == $game->WINNER_WHITE ) {
				$whitehtml = "<span class=\"game_winner\">" . $white->getIdentifier() . "</span>";
			}
			else {
				$whitehtml = $white->getIdentifier();
			}

			if ( $game->getWinner() == $game->WINNER_BLACK ) {
				$blackhtml = "<span class=\"game_winner\">" . $black->getIdentifier() . "</span>";
			}
			else {
				$blackhtml = $black->getIdentifier();
			}

			$whitehtml = '<a href="player_profile_view.php?player='
			              . $white->getIdentifier() . '">' . $whitehtml . '</a>';

			$blackhtml = '<a href="player_profile_view.php?player='
			              . $black->getIdentifier() . '">' . $blackhtml . '</a>';

			$startdate = $game->getStartDate();

			$lastmove  = $game->getLastMove();

			if ( $lastmove ) {
				$lastdate = $lastmove->getDate();
			}
			else {
				$lastdate = $game->getStartDate();
			}

			if ( $startdate ) { // This should always be true
				$starttext = date(DATEFORM_DATE, $startdate);
			}
			else {
				$starttext = "No starting date";
			}

			if ( $lastdate ) {
				$lasttext = date(DATEFORM_FULL, $lastdate);
			}
			else {
				$lasttext = "No move yet";
			}

			$starttext = str_replace(" ", "&nbsp;", $starttext);
			$lasttext  = str_replace(" ", "&nbsp;", $lasttext);

			$turn = $game->getTurnCount();
			$gameid = $game->getId();

			if ( $p_condensed ) {
				$result .= <<<EOT
				<tr>
					<td class="games_data">$whitehtml&nbsp;</td>
					<td class="games_data">$blackhtml&nbsp;</td>
					<td class="games_data" style="text-align: right;">$turn&nbsp;</td>
					<td class="games_data">
					  <a href="$script?gameid=$gameid">[&nbsp;Enter&nbsp;]</a>
					</td>
				</tr>
EOT;
			}
			else {
				$result .= <<<EOT
				<tr>
					<td class="games_data"><img alt="$light" src="$imgroot/$light" />&nbsp;</td>
					<td class="games_data">$whitehtml&nbsp;</td>
					<td class="games_data">$blackhtml&nbsp;</td>
					<td class="games_data">$starttext&nbsp;&nbsp;</td>
					<td class="games_data">$turn&nbsp;</td>
					<td class="games_data">$lasttext&nbsp;&nbsp;</td>
					<td class="games_data">
					  <a href="$script?gameid=$gameid">[&nbsp;Enter&nbsp;]</a>
					</td>
				</tr>
EOT;
			}
		}


		if ( $p_paging && $total_pages > 1 ) {
			$result .= "<tr><td class=\"games_pager\" colspan=\"7\">";

			if ( $gpage > 1 ) {
				$url = mcc_build_url(NULL, GAMES_PAGE_VARNAME, $gpage - 1);
				$result .= "<a href=\"$url\">&lt;</a> &nbsp;";
			}

			$page_first = max(1, $gpage - floor(SHOW_PAGES / 2));
			$page_last  = min($total_pages, $page_first + SHOW_PAGES);

			$result .= "Page &nbsp;";

			for ( $p = $page_first; $p <= $page_last; $p++ ) {
				if ( $gpage == $p ) {
					$result .= "<strong>$p</strong> &nbsp;";
				}
				else {
					$url = mcc_build_url(NULL, GAMES_PAGE_VARNAME, $p);
					$result .= "<a href=\"$url\">$p</a> &nbsp;";
				}
			}

			if ( $gpage < $total_pages ) {
				$url = mcc_build_url(NULL, GAMES_PAGE_VARNAME, $gpage + 1);
				$result .= "<a href=\"$url\">&gt;</a> ";
			}

			$result .= "</td></tr>";
		}

		$result .= "</table>";
	}

	return $result;
}

?>